==> em_celery-0.1.42-py2.py3-none-any.whl: em_celery/tools/spree/amz_offers_update_task_sender.py <==
# -*- coding: utf-8 -*-

import datetime
import json
import time
import random

from kombu import Connection
from dropshipping.utils.utils import is_asin_valid
import dateutil
import dateutil.parser
import redis

from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_celery import logger, get_offer_service
from em_celery.tasks.spapi_update_item_offers_task import spapi_update_item_offers


class AmzOffersUpdateTaskSender():
  def __init__(self, store_code, broker_url, qps, marketplace, condition, ttl, force):
    store = StoreUtil.get_store_by_code(store_code)
    credential = json.loads(store.api_credential)
    self.data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
    self.offer_service = get_offer_service()
    self.broker_url = broker_url
    self.marketplace = marketplace.lower()
    self.qps = qps
    self.condition = condition
    self.ttl = ttl
    self.force = force
    self.connection = Connection(broker_url)
    self.queue = 'SpapiItemOffersUpdate_{}'.format(marketplace.upper())
    self.offer_type = 'lowest_offer_listings'
    self.last_send_time = None
    self.r = redis.Redis.from_url(broker_url)
    self.max_tasks_cnt = 5000

  def run(self):
    cnt = self.tasks_cnt()
    if cnt > self.max_tasks_cnt and not self.force:
      logger.info('[TasksProcessing] %s', cnt)
      return

    if cnt:
      self.ttl += 36

    asins_buf = []
    batch_size = 500
    for sp in self.data_source.get_amz_products(self.marketplace):
      asin = sp.get('source_product_id', None) or sp.get('asin', None)
      if not asin:
        continue

      if not is_asin_valid(asin):
        continue

      asins_buf.append(asin)
      if len(asins_buf) < batch_size:
        continue

      self.process_products(asins_buf)
      asins_buf = []

    if asins_buf:
      self.process_products(asins_buf)
      asins_buf = []

  def process_products(self, asins):
    if self.force:
      asins_without_offer = list(asins)
    else:
      now = datetime.datetime.utcnow()
      offer_expire_time = now - datetime.timedelta(hours=self.ttl)

      offers = dict()
      result = self.offer_service.search_offers(
        self.offer_type, asins, self.marketplace, self.condition)
      while isinstance(result, dict) and 'hits' in result:
        result = result['hits']
      if isinstance(result, list):
        for offer in result:
          if not offer:
            continue

          offers[offer['_source']['asin']] = offer['_source']
      else:
        offers = result
      asins_without_offer = dict()
      for asin in asins:
        if asin not in offers or not offers[asin]:
          asins_without_offer[asin] = None
          continue

        offer = offers[asin]
        offer_time_s = offer.get('time', None)
        if not offer_time_s:
          asins_without_offer[asin] = None
          continue

        try:
          offer_time = dateutil.parser.parse(offer_time_s)

          # Offer expired
          if offer_time < offer_expire_time:
            asins_without_offer[asin] = None
            continue
        except Exception as e:
          asins_without_offer[asin] = None

        logger.debug('[ASINHasOffer] %s', asin)

      asins_without_offer = list(asins_without_offer.keys())

    random.shuffle(asins_without_offer)
    chunks = [asins_without_offer[x:x + 20] for x in range(0, len(asins_without_offer), 20)]
    for chunk in chunks:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          logger.debug("Waiting %.3fs to send next message", wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      spapi_update_item_offers.apply_async(
        args=(self.marketplace, chunk, self.condition),
        queue=self.queue,
        connection=self.connection)
      logger.debug(
        'Added spapi_update_item_offers(%s, %s, %s)',
        self.marketplace, chunk, self.condition)

  def tasks_cnt(self):
    cnt = 0
    try:
      cnt = self.r.llen(self.queue)
    except:
      pass

    return cnt

==> em_celery-0.1.46-py2.py3-none-any.whl: em_celery/tools/spree/amz_offers_update_task_sender.py <==
# -*- coding: utf-8 -*-

import datetime
import json
import time
import random

from kombu import Connection
from dropshipping.utils.utils import is_asin_valid
import dateutil
import dateutil.parser
import redis

from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_celery import logger, get_offer_service
from em_celery.tasks.spapi_update_item_offers_task import spapi_update_item_offers


class AmzOffersUpdateTaskSender():
  def __init__(self, store_code, broker_url, qps, marketplace, condition, ttl, force):
    store = StoreUtil.get_store_by_code(store_code)
    credential = json.loads(store.api_credential)
    self.data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
    self.offer_service = get_offer_service()
    self.broker_url = broker_url
    self.marketplace = marketplace.lower()
    self.qps = qps
    self.condition = condition
    self.ttl = ttl
    self.force = force
    self.connection = Connection(broker_url)
    self.queue = 'SpapiItemOffersUpdate_{}'.format(marketplace.upper())
    self.offer_type = 'lowest_offer_listings'
    self.last_send_time = None
    self.r = redis.Redis.from_url(broker_url)
    self.max_tasks_cnt = 5000
    self.stats = {'expired_cnt': 0, 'alive_cnt': 0}

  def run(self):
    cnt = self.tasks_cnt()
    if cnt > self.max_tasks_cnt and not self.force:
      logger.info('[TasksProcessing] %s', cnt)
      return self.stats

    asins_buf = []
    batch_size = 1000
    for sp in self.data_source.get_amz_products(self.marketplace):
      asin = sp.get('source_product_id', None) or sp.get('asin', None)
      if not asin:
        continue

      if not is_asin_valid(asin):
        continue

      asins_buf.append(asin)
      if len(asins_buf) < batch_size:
        continue

      expired_cnt = self.process_products(asins_buf)
      self.stats['expired_cnt'] += expired_cnt
      self.stats['alive_cnt'] += len(asins_buf) - expired_cnt
      asins_buf = []

    if asins_buf:
      expired_cnt = self.process_products(asins_buf)
      self.stats['expired_cnt'] += expired_cnt
      self.stats['alive_cnt'] += len(asins_buf) - expired_cnt
      asins_buf = []

    return self.stats

  def process_products(self, asins):
    if self.force:
      asins_without_offer = list(asins)
    else:
      now = datetime.datetime.utcnow()
      offer_expire_time = now - datetime.timedelta(hours=self.ttl)

      offers = dict()
      result = self.offer_service.search_offers(
        self.offer_type, asins, self.marketplace, self.condition)
      while isinstance(result, dict) and 'hits' in result:
        result = result['hits']
      if isinstance(result, list):
        for offer in result:
          if not offer:
            continue

          offers[offer['_source']['asin']] = offer['_source']
      else:
        offers = result
      asins_without_offer = dict()
      for asin in asins:
        if asin not in offers or not offers[asin]:
          asins_without_offer[asin] = None
          continue

        offer = offers[asin]
        offer_time_s = offer.get('time', None)
        if not offer_time_s:
          asins_without_offer[asin] = None
          continue

        try:
          offer_time = dateutil.parser.parse(offer_time_s)

          # Offer expired
          if offer_time < offer_expire_time:
            asins_without_offer[asin] = None
            continue
        except Exception as e:
          asins_without_offer[asin] = None

        logger.debug('[ASINHasOffer] %s', asin)

      asins_without_offer = list(asins_without_offer.keys())

    random.shuffle(asins_without_offer)
    chunks = [asins_without_offer[x:x + 20] for x in range(0, len(asins_without_offer), 20)]
    for chunk in chunks:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          logger.debug("Waiting %.3fs to send next message", wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      spapi_update_item_offers.apply_async(
        args=(self.marketplace, chunk, self.condition),
        queue=self.queue,
        connection=self.connection)
      logger.debug(
        'Added spapi_update_item_offers(%s, %s, %s)',
        self.marketplace, chunk, self.condition)

    return len(asins_without_offer)

  def tasks_cnt(self):
    cnt = 0
    try:
      cnt = self.r.llen(self.queue)
    except:
      pass

    return cnt
